<?php
/**
 * Acumbamail Signup Forms - Main file
 *
 * @category   	Module
 * @author     	Acumbamail <jesus.botella@acumbamail.com>
 * @copyright  	2014 Acumbamail
 * @version   	1.1
 * @link       	http://www.acumbamail.com/
 */

// Security
if (!defined('_PS_VERSION_'))
    exit;

require_once(__DIR__ . '/api/acumbamail.class.php');

class Acumbamail extends Module {

    public function __construct()
    {
        $this->author = 'Acumbamail';
        $this->name = 'acumbamail';
        $this->version = '1.1';

        $this->ps_versions_compliancy['min'] = '1.5';
        $this->ps_versions_compliancy['max'] = '1.7';
        $this->need_instance = 0;

        parent::__construct();

        // Name in the modules list
        $this->displayName = $this->l('Acumbamail Signup Forms');
        $this->description = $this->l('Incluye fácilmente un formulario para que los usuarios de tu tienda se suscriban a tu newsletter de Acumbamail');
        $this->confirmUninstall = $this->l('¿Estás seguro de que deseas borrar este módulo?');

        $this->api = new AcumbamailAPI(0, Configuration::get('ACMB_AUTH_TOKEN'));
        ini_set("log_errors", 1);
        ini_set("error_log", "/tmp/php-error.log");
    }

    function hookLeftColumn($params) {
        if (Configuration::get('ACMB_FORM_TO_SHOW') != '') {
            $form_details = $this->api->getFormDetails(Configuration::get('ACMB_FORM_TO_SHOW'));
            $this->context->smarty->assign('acmb_form_details', $form_details);
        }

        return $this->display(__FILE__, 'views/templates/front/frontend.tpl');
    }

    function hookRightColumn($params) {
        return $this->hookLeftColumn($params);
    }

    function hookFooter($params) {
        return $this->hookLeftColumn($params);
    }

    public function hookActionValidateOrder($params) {
        $list_id = Configuration::get('ACMB_CLIENTS_LIST');
        $active_subscribers_status = 0;
        $subscribers = $this->api->getSubscribers($list_id, $active_subscribers_status);
        $already_subscribed = false;
        foreach (array_keys($subscribers) as $subscriber_key) {
            if ($subscribers[$subscriber_key]['email'] === $params['customer']->email) {
                $already_subscribed = true;
            }
        }

        if (!$already_subscribed) {
            $subscriber_fields = array();
            $subscriber_fields['email'] = $params['customer']->email;
            $this->api->addSubscriber($list_id, $subscriber_fields);
        }
    }
    
    public function install() {
        if (!parent::install() || !$this->registerHook('displayLeftColumn') || !$this->registerHook('actionValidateOrder'))
            return false;
        return true;
    }

    public function uninstall(){
        $this->flush_preferences();

        if (!parent::uninstall())
            return false;
        return true;
    }

    public function getContent() {
        $output = '<h2>'.$this->displayName.'</h2>';

        if (isset($errors) && count($errors)) {
            $output .= $this->displayError(implode('<br />', $errors));
        }
        
        //Submit del primer formulario (Configuración de Datos de Acumbamail)
        if (Tools::isSubmit('submit_form'))
        {
            $this->formToSetAuthTokenSubmit();
            $output .= $this->formToShowInShopSubmit();
            $output .= $this->formToSubscribeClientsSubmit();
        }

        if (Tools::isSubmit('reset_form')) {
            $this->flush_preferences();
        }

        return $output.$this->displayForm();
    }

    public function displayForm()
    {

        if(isset($errors))
            $this->context->smarty->assign('errors', $errors);
        else
            $errors = array();

        if (Configuration::get('ACMB_AUTH_TOKEN') != '') {
            $query_lists = $this->api->getLists();
            error_log("After calling query list, getting forms: ".Configuration::get('ACMB_SINGLE_LIST'));
            if ($query_lists != '') {
                $query_forms = $this->api->getForms(Configuration::get('ACMB_SINGLE_LIST'));
                $this->context->smarty->assign('listas', $query_lists);
                $this->context->smarty->assign('forms', $query_forms);
            }
            else {
                array_push($errors, $this->l("Token de Autenticación incorrecto: ". Configuration::get('ACMB_AUTH_TOKEN')));
                $this->flush_preferences();
            }
        }
        
        if(Configuration::get("ACMB_SINGLE_LIST")!='' and Configuration::get("ACMB_FORM_TO_SHOW") != '') {
            $this->context->smarty->assign('acmb_form_details', $this->api->getFormDetails(Configuration::get('ACMB_FORM_TO_SHOW')));
        }

        $this->context->smarty->assign('request_uri', Tools::safeOutput($_SERVER['REQUEST_URI']));
        $this->context->smarty->assign('path', $this->_path);
        $this->context->smarty->assign('submitName', 'submit'.ucfirst($this->name));
        $this->context->smarty->assign('errors', $errors);
        $this->context->smarty->assign('acmb_auth_token', Configuration::get('ACMB_AUTH_TOKEN'));
        $this->context->smarty->assign('acmb_single_list', Configuration::get('ACMB_SINGLE_LIST'));
        $this->context->smarty->assign('acmb_form_to_show', Configuration::get('ACMB_FORM_TO_SHOW'));
        
        $selected_list["single"] = Configuration::get('ACMB_SINGLE_LIST');
        $this->context->smarty->assign('selected_list', $selected_list);
        
        $buyers_list["single"] = Configuration::get('ACMB_CLIENTS_LIST');
        $this->context->smarty->assign('buyers_list', $buyers_list);
        
        $selected_form["single"] = Configuration::get("ACMB_FORM_TO_SHOW");
        $this->context->smarty->assign('selected_form', $selected_form);

        // You can return html, but I prefer this new version: use smarty in admin, :)
        return $this->display(__FILE__, 'views/templates/admin/configure.tpl');
    }

    private function formToSetAuthTokenSubmit() {
        $acumba_auth_token = Tools::getValue('acmb_auth_token');
        if($acumba_auth_token != '') {
            Configuration::updateValue('ACMB_AUTH_TOKEN',$acumba_auth_token);
            $this->api->setAuthToken($acumba_auth_token);
        }
        else {
            $this->flush_preferences();
        }
    }

    private function formToShowInShopSubmit() {
        $output_string = '';

        if(Tools::getValue('list-general')!='') {
            $single_list = Tools::getValue('list-general');
            Configuration::updateValue('ACMB_SINGLE_LIST', $single_list);
            $form_to_show = Tools::getValue('forms-general');
            if ($form_to_show != '') {
                Configuration::updateValue("ACMB_FORM_TO_SHOW", $form_to_show);
                $output_string .= $this->displayConfirmation('El formulario será mostrado en su tienda');
            }
        }
        else {
            Configuration::deleteByName("ACMB_SINGLE_LIST");
            Configuration::deleteByName("ACMB_FORM_TO_SHOW");
        }

        return $output_string;
    }

    private function formToSubscribeClientsSubmit() {
        $output_string = '';
        $client_subscription_list = Tools::getValue('list-subscription');
        if($client_subscription_list != '') {
            Configuration::updateValue("ACMB_CLIENTS_LIST", $client_subscription_list);
            $output_string .= $this->displayConfirmation('Sus compradores se suscribirán automáticamente a la lista');
            if (Tools::getValue('sync_newsletter_clients')) {
                $this->subscribeNewsletterClients($client_subscription_list);
            }
        }
        else {
            Configuration::deleteByName("ACMB_CLIENTS_LIST");
        }
        return $output_string;
    }

    private function subscribeNewsletterClients($client_subscription_list) {
        $difference = $this->getSubscriptionsDifferences($client_subscription_list);
        foreach ($difference as $client_to_subscribe) {
            $merge_fields['email'] = $client_to_subscribe['email'];
            $this->api->addSubscriber($client_subscription_list, $merge_fields);
        }
    }

    private function getSubscriptionsDifferences($client_subscription_list) {
        $newsletter_subscriptions = $this->getNewsletterSubscriptions();
        $acumbamail_subscriptions = $this->api->getSubscribers($client_subscription_list, 0);
        $differences = array();
        foreach ($newsletter_subscriptions as $newsletter) {
            $found = false;
            foreach ($acumbamail_subscriptions as $acumbamail) {
                if ($acumbamail['email'] == $newsletter['email']) {
                    $found = true;
                }
            }
            
            if (!$found) {
                array_push($differences, $newsletter);
            }
        }
        
        return $differences;
    }

    private function getNewsletterSubscriptions() {
        $sql = new DbQuery();
        $sql->select('email');
        $sql->from('emailsubscription');
        try {
            $newsletter_subscriptions = Db::getInstance()->executeS($sql);
        }
        catch (Exception $e) {
            $newsletter_subscriptions = array();
        }
        
        return $newsletter_subscriptions;
    }
    
    private function flush_preferences() {
        //Eliminamos cadenas de configuración
        Configuration::deleteByName("ACMB_AUTH_TOKEN");
        Configuration::deleteByName("ACMB_SINGLE_LIST");
        Configuration::deleteByName("ACMB_CLIENTS_LIST");
        Configuration::deleteByName("ACMB_FORM_TO_SHOW");
    }
}
